/// @addtogroup dynset
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file dynsetlib.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file constitutes a part of the CAPD library, 
// distributed under the terms of the GNU General Public License.
// Consult  http://capd.wsb-nlu.edu.pl/ for details. 

/* Author: Daniel Wilczak 2005 */

#ifndef _CAPD_DYNSET_DYNSETLIB_H_ 
#define _CAPD_DYNSET_DYNSETLIB_H_ 

#include "capd/interval/DoubleInterval.h"
#include "capd/vectalg/vectalgLib.h"

#include "capd/dynset/BallSet.h"
#include "capd/dynset/EllipsoidSet.h"
#include "capd/dynset/FlowballSet.h"
#include "capd/dynset/Intv2Set.h"
#include "capd/dynset/IntervalSet.h"
#include "capd/dynset/Pped2Set.h"
#include "capd/dynset/PpedSet.h"
#include "capd/dynset/Rect2Set.h"
#include "capd/dynset/RectSet.h"

#include "capd/dynset/C1Rect.h"
#include "capd/dynset/C1Rect2.h"
#include "capd/dynset/C11Rect2.h"
#include "capd/dynset/C2Rect2.h"
#include "capd/dynset/CnRect2.h"

typedef interval (*v_form)(IVector &);

typedef capd::dynset::C0Set<IMatrix> C0Set;
typedef capd::dynset::BallSet<IMatrix> BallSet;
typedef capd::dynset::EllipsoidSet<DVector,IMatrix> EllipsoidSet;
typedef capd::dynset::FlowballSet<IMatrix> FlowballSet;
typedef capd::dynset::Intv2Set<IMatrix> Intv2Set;
typedef capd::dynset::IntervalSet<IMatrix> IntervalSet;
typedef capd::dynset::Pped2Set<IMatrix> Pped2Set;
typedef capd::dynset::PpedSet<IMatrix> PpedSet;
typedef capd::dynset::Rect2Set<IMatrix> Rect2Set;
typedef capd::dynset::RectSet<IMatrix> RectSet;

typedef capd::dynset::C1Set<IMatrix> C1Set;
typedef capd::dynset::C1Rect<IMatrix> C1Rect;
typedef capd::dynset::C1Rect2<IMatrix> C1Rect2;
typedef capd::dynset::C11Rect2<IMatrix> C11Rect2;

typedef capd::dynset::C2Set<IMatrix> C2Set;
typedef capd::dynset::C2Rect2<IMatrix> C2Rect2;

typedef capd::dynset::CnSet<IMatrix> CnSet;
typedef capd::dynset::CnRect2<IMatrix> CnRect2;




typedef capd::dynset::C0Set<IMatrixMD> C0SetMD;
typedef capd::dynset::BallSet<IMatrixMD> BallSetMD;
typedef capd::dynset::EllipsoidSet<DVectorMD,IMatrixMD> EllipsoidSetMD;
typedef capd::dynset::FlowballSet<IMatrixMD> FlowballSetMD;
typedef capd::dynset::Intv2Set<IMatrixMD> Intv2SetMD;
typedef capd::dynset::IntervalSet<IMatrixMD> IntervalSetMD;
typedef capd::dynset::Pped2Set<IMatrixMD> Pped2SetMD;
typedef capd::dynset::PpedSet<IMatrixMD> PpedSetMD;
typedef capd::dynset::Rect2Set<IMatrixMD> Rect2SetMD;
typedef capd::dynset::RectSet<IMatrixMD> RectSetMD;

typedef capd::dynset::C1Set<IMatrixMD> C1SetMD;
typedef capd::dynset::C1Rect<IMatrixMD> C1RectMD;
typedef capd::dynset::C1Rect2<IMatrixMD> C1Rect2MD;
typedef capd::dynset::C11Rect2<IMatrixMD> C11Rect2MD;

typedef capd::dynset::C2Set<IMatrixMD> C2SetMD;
typedef capd::dynset::C2Rect2<IMatrixMD> C2Rect2MD;

typedef capd::dynset::CnSet<IMatrixMD> CnSetMD;
typedef capd::dynset::CnRect2<IMatrixMD> CnRect2MD;

#ifdef __HAVE_MPFR__

  typedef capd::dynset::C0Set<IMpMatrix> MpC0Set;
  typedef capd::dynset::BallSet<IMpMatrix> MpBallSet;
  typedef capd::dynset::EllipsoidSet<MpVector,IMpMatrix> MpEllipsoidSet;
  typedef capd::dynset::FlowballSet<IMpMatrix> MpFlowballSet;
  typedef capd::dynset::Intv2Set<IMpMatrix> MpIntv2Set;
  typedef capd::dynset::IntervalSet<IMpMatrix> MpIntervalSet;
  typedef capd::dynset::Pped2Set<IMpMatrix> MpPped2Set;
  typedef capd::dynset::PpedSet<IMpMatrix> MpPpedSet;
  typedef capd::dynset::Rect2Set<IMpMatrix> MpRect2Set;
  typedef capd::dynset::RectSet<IMpMatrix> MpRectSet;

  typedef capd::dynset::C1Set<IMpMatrix> MpC1Set;
  typedef capd::dynset::C1Rect<IMpMatrix> MpC1Rect;
  typedef capd::dynset::C1Rect2<IMpMatrix> MpC1Rect2;
  typedef capd::dynset::C11Rect2<IMpMatrix> MpC11Rect2;

  typedef capd::dynset::C2Set<IMpMatrix> MpC2Set;
  typedef capd::dynset::C2Rect2<IMpMatrix> MpC2Rect2;

  typedef capd::dynset::CnSet<IMpMatrix> MpCnSet;
  typedef capd::dynset::CnRect2<IMpMatrix> MpCnRect2;




  typedef capd::dynset::C0Set<IMpMatrixMD> MpC0SetMD;
  typedef capd::dynset::BallSet<IMpMatrixMD> MpBallSetMD;
  typedef capd::dynset::EllipsoidSet<MpVectorMD,IMpMatrixMD> MpEllipsoidSetMD;
  typedef capd::dynset::FlowballSet<IMpMatrixMD> MpFlowballSetMD;
  typedef capd::dynset::Intv2Set<IMpMatrixMD> MpIntv2SetMD;
  typedef capd::dynset::IntervalSet<IMpMatrixMD> MpIntervalSetMD;
  typedef capd::dynset::Pped2Set<IMpMatrixMD> MpPped2SetMD;
  typedef capd::dynset::PpedSet<IMpMatrixMD> MpPpedSetMD;
  typedef capd::dynset::Rect2Set<IMpMatrixMD> MpRect2SetMD;
  typedef capd::dynset::RectSet<IMpMatrixMD> MpRectSetMD;

  typedef capd::dynset::C1Set<IMpMatrixMD> MpC1SetMD;
  typedef capd::dynset::C1Rect<IMpMatrixMD> MpC1RectMD;
  typedef capd::dynset::C1Rect2<IMpMatrixMD> MpC1Rect2MD;
  typedef capd::dynset::C11Rect2<IMpMatrixMD> MpC11Rect2MD;

  typedef capd::dynset::C2Set<IMpMatrixMD> MpC2SetMD;
  typedef capd::dynset::C2Rect2<IMpMatrixMD> MpC2Rect2MD;

  typedef capd::dynset::CnSet<IMpMatrixMD> MpCnSetMD;
  typedef capd::dynset::CnRect2<IMpMatrixMD> MpCnRect2MD;

#endif

#endif // _CAPD_DYNSET_DYNSETLIB_H_ 

/// @}
