/// @addtogroup dynsys
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file Taylor.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file constitutes a part of the CAPD library, 
// distributed under the terms of the GNU General Public License.
// Consult  http://capd.wsb-nlu.edu.pl/ for details. 

/* Author: Daniel Wilczak 2001-2005 */

#ifndef _CAPD_DYNSYS_TAYLOR_H_ 
#define _CAPD_DYNSYS_TAYLOR_H_ 

#include <string>
#include "capd/vectalg/Norm.h"
#include "capd/dynsys/C1DynSys.h"
#include "capd/dynsys/BasicTaylor.h"

namespace capd{
namespace dynsys{

template<typename MapT>
class Taylor:   public virtual C1DynSys<typename MapT::MatrixType>, public virtual BasicTaylor<MapT>
{
public:
  typedef MapT MapType;
  typedef typename MapT::FunctionType FunctionType;
  typedef typename MapType::MatrixType MatrixType;
  typedef typename MatrixType::RowVectorType VectorType;
  typedef typename MatrixType::ScalarType ScalarType;
  typedef capd::vectalg::Norm<VectorType,MatrixType> NormType;

  Taylor(MapType& vField, int order, const ScalarType& step);

  VectorType Phi(const VectorType& iv) const;
  MatrixType JacPhi(const VectorType& iv) const;
  VectorType Remainder(const VectorType& iv) const;

  void JacRemainder(
         const VectorType &vecEnclosure,
         const MatrixType &jacEnclosure,
         VectorType &Remainder,
         MatrixType &jRemainder
      ) const;

  VectorType enclosure(const VectorType& x, int* found) const;
  VectorType enclosure(const VectorType &x) const;
  MatrixType jacEnclosure(const VectorType& enc, const NormType& norm) const;

  using BasicTaylor<MapType>::getField;
  using BasicTaylor<MapType>::getOrder;
  using BasicTaylor<MapType>::getStep;
  using BasicTaylor<MapType>::getCoeff;
  using BasicTaylor<MapType>::getMatrixCoeff;

  using BasicTaylor<MapType>::setStep;
  using BasicTaylor<MapType>::setOrder;


protected:
  void operator=(const Taylor& ){}
  Taylor(const Taylor& t) : BasicTaylor<MapT>(t){}

  using BasicTaylor<MapType>::computeMatrixCoeff;
  using BasicTaylor<MapType>::m_vField;
  using BasicTaylor<MapType>::m_order;
  using BasicTaylor<MapType>::m_step;
  using BasicTaylor<MapType>::m_coeff;
  using BasicTaylor<MapType>::m_matrixCoeff;
  using BasicTaylor<MapType>::m_dimension;
  using BasicTaylor<MapType>::m_F;
};


// --------------- inline definitions -----------------

//###########################################################//

template<typename MapType>
inline typename Taylor<MapType>::VectorType Taylor<MapType>::Phi(const VectorType& iv) const
{
  return BasicTaylor<MapType>::operator()(iv);
}

}} // namespace capd::dynsys

#endif // _CAPD_DYNSYS_TAYLOR_H_ 

/// @}
