/// @addtogroup map
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file Node_explog.hpp
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file constitutes a part of the CAPD library, 
// distributed under the terms of the GNU General Public License.
// Consult  http://capd.wsb-nlu.edu.pl/ for details. 

/* Author: Daniel Wilczak 2004-2005 */

#ifndef _CAPD_MAP_NODE_EXPLOG_HPP_ 
#define _CAPD_MAP_NODE_EXPLOG_HPP_ 

namespace capd{
namespace map{

// ----------------------- constructors --------------------------

template<typename ScalarType>
ExpNode<ScalarType>::ExpNode(Node<ScalarType> *a_left, int a_order)
  : Node<ScalarType>(a_order,a_left,0)
{
  value = new ScalarType[m_order];
  iterator b=begin(), e=end();
  while(b!=e)
  {
    *b = ScalarType(0);
    ++b;
  }
  if(left) ++(left->m_links);
}

template<typename ScalarType>
LogNode<ScalarType>::LogNode(Node<ScalarType> *a_left, int a_order)
  : Node<ScalarType>(a_order,a_left,0)
{
  value = new ScalarType[m_order];
  iterator b=begin(), e=end();
  while(b!=e)
  {
    *b = ScalarType(0);
    ++b;
  }
  if(left) ++(left->m_links);
}

// ---------------------- destructors ----------------------------

template<typename ScalarType>
ExpNode<ScalarType>::~ExpNode()
{
  if( !( --(left->m_links)) ) 
    delete left;
  delete []value;
}

template<typename ScalarType>
LogNode<ScalarType>::~LogNode()
{
  if( !( --(left->m_links)) ) 
    delete left;
  delete []value;
}

// ----------------------- operators -----------------------------

template<typename ScalarType>
ScalarType& ExpNode<ScalarType>::operator()(int i)
{
  if(i<=m_maxComputedDerivative)
    return value[i];
  m_maxComputedDerivative = i;

  (*left)(i);

  if(i==0)
    return value[i] = exp(left->value[0]);

  ScalarType result(0);
  for(int j=0;j<i;++j)
    result += ScalarType(i-j) * value[j] * left->value[i-j];

  return value[i] = result/ScalarType(i);
}

template<typename ScalarType>
ScalarType& LogNode<ScalarType>::operator()(int i)
{
  if(i<=m_maxComputedDerivative)
    return value[i];
  m_maxComputedDerivative = i;

  (*left)(i);

  if(i==0) return value[i] = log(left->value[0]);

  ScalarType result(0.);
  for(int j=0;j<i;++j)
    result+= ScalarType(i-j) * left->value[j] * value[i-j];

  return value[i] = (left->value[i] - result/ScalarType(i)) / left->value[0];
}

// -------------------------- setOrder -----------------------------

template<typename ScalarType>
void ExpNode<ScalarType>::setOrder(int a_order, ScalarType *a_val)
{
  if(m_order==a_order) return;
  delete [] value;

  m_order = a_order;
  value = new ScalarType[m_order];

  if(left) left->setOrder(m_order,a_val);
  if(right) right->setOrder(m_order,a_val);
}

template<typename ScalarType>
void LogNode<ScalarType>::setOrder(int a_order, ScalarType *a_val)
{
  if(m_order==a_order) return;
  delete [] value;

  m_order = a_order;
  value = new ScalarType[m_order];

  if(left) left->setOrder(m_order,a_val);
  if(right) right->setOrder(m_order,a_val);
}

}} // namespace capd::map

#endif // _CAPD_MAP_NODE_EXPLOG_HPP_ 

/// @}
