/// @addtogroup vectalg
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file IMpMatrix.cpp
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file constitutes a part of the CAPD library, 
// distributed under the terms of the GNU General Public License.
// Consult  http://capd.wsb-nlu.edu.pl/ for details. 


#ifdef __HAVE_MPFR__

#include "capd/interval/DoubleInterval.h"
#include "capd/vectalg/Dimension.h"
#include "capd/vectalg/Vector.hpp"
#include "capd/vectalg/Matrix.hpp"
#include "capd/vectalg/vectalgLib.h"

namespace capd{ 
  namespace vectalg{


template class Matrix<MpInterval,DIM,DIM>;
template class RowVector<MpInterval,DIM>;
template class ColumnVector<MpInterval,DIM>;

template Matrix<MpInterval,DIM,DIM> abs <MpInterval,DIM,DIM> (const Matrix<MpInterval,DIM,DIM>&);
template Matrix<MpInterval,DIM,DIM> operator- <MpInterval,DIM,DIM> (const Matrix<MpInterval,DIM,DIM>&);
template Matrix<MpInterval,DIM,DIM> operator+ <MpInterval,DIM,DIM> (const Matrix<MpInterval,DIM,DIM>&, const Matrix<MpInterval,DIM,DIM>&);
template Matrix<MpInterval,DIM,DIM> operator- <MpInterval,DIM,DIM> (const Matrix<MpInterval,DIM,DIM>&, const Matrix<MpInterval,DIM,DIM>&);
template Matrix<MpInterval,DIM,DIM> operator* <MpInterval,DIM,DIM> (const Matrix<MpInterval,DIM,DIM>&, const Matrix<MpInterval,DIM,DIM>&);
template Vector<MpInterval,DIM> operator* <MpInterval,DIM,DIM> (const Matrix<MpInterval,DIM,DIM>&, const Vector<MpInterval,DIM>&);
template Matrix<MpInterval,DIM,DIM> operator* <MpInterval,DIM,DIM> (const Matrix<MpInterval,DIM,DIM>&, const MpInterval&);
template Matrix<MpInterval,DIM,DIM> operator* <MpInterval,DIM,DIM> (const MpInterval&, const Matrix<MpInterval,DIM,DIM>&);
template Matrix<MpInterval,DIM,DIM> operator/ <MpInterval,DIM,DIM> (const Matrix<MpInterval,DIM,DIM>&, const MpInterval&);
template Matrix<MpInterval,DIM,DIM> operator+ <MpInterval,DIM,DIM> (const Matrix<MpInterval,DIM,DIM>&, const MpInterval&);
template Matrix<MpInterval,DIM,DIM> operator- <MpInterval,DIM,DIM> (const Matrix<MpInterval,DIM,DIM>&, const MpInterval&);
template bool operator< <MpInterval,DIM,DIM> (const Matrix<MpInterval,DIM,DIM>&, const Matrix<MpInterval,DIM,DIM>&);
template bool operator> <MpInterval,DIM,DIM> (const Matrix<MpInterval,DIM,DIM>&, const Matrix<MpInterval,DIM,DIM>&);
template bool operator<= <MpInterval,DIM,DIM> (const Matrix<MpInterval,DIM,DIM>&, const Matrix<MpInterval,DIM,DIM>&);
template bool operator>= <MpInterval,DIM,DIM> (const Matrix<MpInterval,DIM,DIM>&, const Matrix<MpInterval,DIM,DIM>&);
template Matrix<MpInterval,DIM,DIM> Transpose <MpInterval,DIM,DIM> (const Matrix<MpInterval,DIM,DIM>&);
template std::ostream &operator<< <MpInterval,DIM,DIM> (std::ostream&, const Matrix<MpInterval,DIM,DIM>&);
template std::istream &operator>> <MpInterval,DIM,DIM> (std::istream&, Matrix<MpInterval,DIM,DIM>&);

// intervalMatrix

template void split<IMpMatrix>(IMpMatrix&, IMpMatrix&);
template IMpMatrix midMatrix<IMpMatrix>(const IMpMatrix&);
template bool intersection<IMpMatrix>(const IMpMatrix&,const IMpMatrix&, IMpMatrix&);


// RowVector

template Vector<MpInterval,DIM> operator+<MpInterval,DIM>(
      const Vector<MpInterval,DIM>&,
      const RowVector<MpInterval,DIM>&
   );
template Vector<MpInterval,DIM> operator+<MpInterval,DIM>(
      const RowVector<MpInterval,DIM>&,
      const Vector<MpInterval,DIM>&
   );
template Vector<MpInterval,DIM> operator+<MpInterval,DIM>(
      const RowVector<MpInterval,DIM>&,
      const RowVector<MpInterval,DIM>&
   );

template Vector<MpInterval,DIM> operator-<MpInterval,DIM>(
      const Vector<MpInterval,DIM>&,
      const RowVector<MpInterval,DIM>&
   );
template Vector<MpInterval,DIM> operator-<MpInterval,DIM>(
      const RowVector<MpInterval,DIM>&,
      const Vector<MpInterval,DIM>&
   );
template Vector<MpInterval,DIM> operator-<MpInterval,DIM>(
      const RowVector<MpInterval,DIM>&,
      const RowVector<MpInterval,DIM>&
   );

template MpInterval operator*<MpInterval,DIM>(
      const Vector<MpInterval,DIM>&,
      const RowVector<MpInterval,DIM>&
   );
template MpInterval operator*<MpInterval,DIM>(
      const RowVector<MpInterval,DIM>&,
      const Vector<MpInterval,DIM>&
   );
template MpInterval operator*<MpInterval,DIM>(
      const RowVector<MpInterval,DIM>&,
      const RowVector<MpInterval,DIM>&
   );

template Vector<MpInterval,DIM> operator-<MpInterval,DIM>(const RowVector<MpInterval,DIM>&);
template Vector<MpInterval,DIM> operator*<MpInterval,DIM,DIM>(
      const Matrix<MpInterval,DIM,DIM>&, const RowVector<MpInterval,DIM>&
   );

template Vector<MpInterval,DIM> operator*<MpInterval,DIM>(const MpInterval&, const RowVector<MpInterval,DIM>&);
template Vector<MpInterval,DIM> operator*<MpInterval,DIM>(const RowVector<MpInterval,DIM>&, const MpInterval&);
template Vector<MpInterval,DIM> operator/<MpInterval,DIM>(const RowVector<MpInterval,DIM>&, const MpInterval&);

template std::ostream& operator<< <MpInterval,DIM>(std::ostream&, const RowVector<MpInterval,DIM>&);


// ColumnVector

template Vector<MpInterval,DIM> operator+<MpInterval,DIM>(
      const Vector<MpInterval,DIM>&,
      const ColumnVector<MpInterval,DIM>&
   );
template Vector<MpInterval,DIM> operator+<MpInterval,DIM>(
      const ColumnVector<MpInterval,DIM>&,
      const Vector<MpInterval,DIM>&
   );
template Vector<MpInterval,DIM> operator+<MpInterval,DIM>(
      const ColumnVector<MpInterval,DIM>&,
      const ColumnVector<MpInterval,DIM>&
   );

template Vector<MpInterval,DIM> operator-<MpInterval,DIM>(
      const Vector<MpInterval,DIM>&,
      const ColumnVector<MpInterval,DIM>&
   );
template Vector<MpInterval,DIM> operator-<MpInterval,DIM>(
      const ColumnVector<MpInterval,DIM>&,
      const Vector<MpInterval,DIM>&
   );
template Vector<MpInterval,DIM> operator-<MpInterval,DIM>(
      const ColumnVector<MpInterval,DIM>&,
      const ColumnVector<MpInterval,DIM>&
   );

template MpInterval operator*<MpInterval,DIM>(
      const Vector<MpInterval,DIM>&,
      const ColumnVector<MpInterval,DIM>&
   );
template MpInterval operator*<MpInterval,DIM>(
      const ColumnVector<MpInterval,DIM>&,
      const Vector<MpInterval,DIM>&
   );
template MpInterval operator*<MpInterval,DIM>(
      const ColumnVector<MpInterval,DIM>&,
      const ColumnVector<MpInterval,DIM>&
   );

template Vector<MpInterval,DIM> operator-<MpInterval,DIM>(const ColumnVector<MpInterval,DIM>&);
template Vector<MpInterval,DIM> operator*<MpInterval,DIM,DIM>(
      const Matrix<MpInterval,DIM,DIM>&,
      const ColumnVector<MpInterval,DIM>&
   );

template Vector<MpInterval,DIM> operator*<MpInterval,DIM>(const MpInterval&, const ColumnVector<MpInterval,DIM>&);
template Vector<MpInterval,DIM> operator*<MpInterval,DIM>(const ColumnVector<MpInterval,DIM>&, const MpInterval&);
template Vector<MpInterval,DIM> operator/<MpInterval,DIM>(const ColumnVector<MpInterval,DIM>&, const MpInterval&);

template std::ostream& operator<< <MpInterval,DIM>(std::ostream&, const ColumnVector<MpInterval,DIM>&);

}}  // end of namespace capd::vectalg

#endif // __HAVE_MPFR__

/// @}
