/// @addtogroup vectalg
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file MpMatrix.cpp
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file constitutes a part of the CAPD library, 
// distributed under the terms of the GNU General Public License.
// Consult  http://capd.wsb-nlu.edu.pl/ for details. 

#ifdef __HAVE_MPFR__

#include "capd/vectalg/vectalgLib.h"
#include "capd/vectalg/Dimension.h"
#include "capd/vectalg/Vector.hpp"
#include "capd/vectalg/ColumnVector.hpp"
#include "capd/vectalg/RowVector.hpp"
#include "capd/vectalg/Matrix.hpp"

namespace capd{ 
  namespace vectalg{


template class Matrix<Real,DIM,DIM>;
template class RowVector<Real,DIM>;
template class ColumnVector<Real,DIM>;

template Matrix<Real,DIM,DIM> abs <Real,DIM,DIM> (const Matrix<Real,DIM,DIM>&);
template Matrix<Real,DIM,DIM> operator- <Real,DIM,DIM> (const Matrix<Real,DIM,DIM>&);
template Matrix<Real,DIM,DIM> operator+ <Real,DIM,DIM> (const Matrix<Real,DIM,DIM>&, const Matrix<Real,DIM,DIM>&);
template Matrix<Real,DIM,DIM> operator- <Real,DIM,DIM> (const Matrix<Real,DIM,DIM>&, const Matrix<Real,DIM,DIM>&);
template Matrix<Real,DIM,DIM> operator* <Real,DIM,DIM> (const Matrix<Real,DIM,DIM>&, const Matrix<Real,DIM,DIM>&);
template Vector<Real,DIM> operator* <Real,DIM,DIM> (const Matrix<Real,DIM,DIM>&, const Vector<Real,DIM>&);
template Matrix<Real,DIM,DIM> operator* <Real,DIM,DIM> (const Matrix<Real,DIM,DIM>&, const Real&);
template Matrix<Real,DIM,DIM> operator* <Real,DIM,DIM> (const Real&, const Matrix<Real,DIM,DIM>&);
template Matrix<Real,DIM,DIM> operator/ <Real,DIM,DIM> (const Matrix<Real,DIM,DIM>&, const Real&);
template Matrix<Real,DIM,DIM> operator+ <Real,DIM,DIM> (const Matrix<Real,DIM,DIM>&, const Real&);
template Matrix<Real,DIM,DIM> operator- <Real,DIM,DIM> (const Matrix<Real,DIM,DIM>&, const Real&);
template bool operator< <Real,DIM,DIM> (const Matrix<Real,DIM,DIM>&, const Matrix<Real,DIM,DIM>&);
template bool operator> <Real,DIM,DIM> (const Matrix<Real,DIM,DIM>&, const Matrix<Real,DIM,DIM>&);
template bool operator<= <Real,DIM,DIM> (const Matrix<Real,DIM,DIM>&, const Matrix<Real,DIM,DIM>&);
template bool operator>= <Real,DIM,DIM> (const Matrix<Real,DIM,DIM>&, const Matrix<Real,DIM,DIM>&);
template Matrix<Real,DIM,DIM> Transpose <Real,DIM,DIM> (const Matrix<Real,DIM,DIM>&);
template std::ostream &operator<< <Real,DIM,DIM> (std::ostream&, const Matrix<Real,DIM,DIM>&);
template std::istream &operator>> <Real,DIM,DIM> (std::istream&, Matrix<Real,DIM,DIM>&);


// RowVector

template Vector<Real,DIM> operator+<Real,DIM>(
      const Vector<Real,DIM>&,
      const RowVector<Real,DIM>&
   );
template Vector<Real,DIM> operator+<Real,DIM>(
      const RowVector<Real,DIM>&,
      const Vector<Real,DIM>&
   );
template Vector<Real,DIM> operator+<Real,DIM>(
      const RowVector<Real,DIM>&,
      const RowVector<Real,DIM>&
   );

template Vector<Real,DIM> operator-<Real,DIM>(
      const Vector<Real,DIM>&,
      const RowVector<Real,DIM>&
   );
template Vector<Real,DIM> operator-<Real,DIM>(
      const RowVector<Real,DIM>&,
      const Vector<Real,DIM>&
   );
template Vector<Real,DIM> operator-<Real,DIM>(
      const RowVector<Real,DIM>&,
      const RowVector<Real,DIM>&
   );

template Real operator*<Real,DIM>(
      const Vector<Real,DIM>&,
      const RowVector<Real,DIM>&
   );
template Real operator*<Real,DIM>(
      const RowVector<Real,DIM>&,
      const Vector<Real,DIM>&
   );
template Real operator*<Real,DIM>(
      const RowVector<Real,DIM>&,
      const RowVector<Real,DIM>&
   );

template Vector<Real,DIM> operator-<Real,DIM>(const RowVector<Real,DIM>&);
template Vector<Real,DIM> operator*<Real,DIM,DIM>(
      const Matrix<Real,DIM,DIM>&, const RowVector<Real,DIM>&
   );

template Vector<Real,DIM> operator*<Real,DIM>(const Real&, const RowVector<Real,DIM>&);
template Vector<Real,DIM> operator*<Real,DIM>(const RowVector<Real,DIM>&, const Real&);
template Vector<Real,DIM> operator/<Real,DIM>(const RowVector<Real,DIM>&, const Real&);

template std::ostream& operator<< <Real,DIM>(std::ostream&, const RowVector<Real,DIM>&);


// ColumnVector

template Vector<Real,DIM> operator+<Real,DIM>(
      const Vector<Real,DIM>&,
      const ColumnVector<Real,DIM>&
   );
template Vector<Real,DIM> operator+<Real,DIM>(
      const ColumnVector<Real,DIM>&,
      const Vector<Real,DIM>&
   );
template Vector<Real,DIM> operator+<Real,DIM>(
      const ColumnVector<Real,DIM>&,
      const ColumnVector<Real,DIM>&
   );

template Vector<Real,DIM> operator-<Real,DIM>(
      const Vector<Real,DIM>&,
      const ColumnVector<Real,DIM>&
   );
template Vector<Real,DIM> operator-<Real,DIM>(
      const ColumnVector<Real,DIM>&,
      const Vector<Real,DIM>&
   );
template Vector<Real,DIM> operator-<Real,DIM>(
      const ColumnVector<Real,DIM>&,
      const ColumnVector<Real,DIM>&
   );

template Real operator*<Real,DIM>(
      const Vector<Real,DIM>&,
      const ColumnVector<Real,DIM>&
   );
template Real operator*<Real,DIM>(
      const ColumnVector<Real,DIM>&,
      const Vector<Real,DIM>&
   );
template Real operator*<Real,DIM>(
      const ColumnVector<Real,DIM>&,
      const ColumnVector<Real,DIM>&
   );

template Vector<Real,DIM> operator-<Real,DIM>(const ColumnVector<Real,DIM>&);
template Vector<Real,DIM> operator*<Real,DIM,DIM>(
      const Matrix<Real,DIM,DIM>&,
      const ColumnVector<Real,DIM>&
   );

template Vector<Real,DIM> operator*<Real,DIM>(const Real&, const ColumnVector<Real,DIM>&);
template Vector<Real,DIM> operator*<Real,DIM>(const ColumnVector<Real,DIM>&, const Real&);
template Vector<Real,DIM> operator/<Real,DIM>(const ColumnVector<Real,DIM>&, const Real&);

template std::ostream& operator<< <Real,DIM>(std::ostream&, const ColumnVector<Real,DIM>&);


}}  // end of namespace capd::vectalg

#endif // __HAVE_MPFR__

/// @}
